<!doctype html>
<html>
  <head>
    <title>{{ pagetitle }}</title>
    <link rel="stylesheet" href="/static/pinnwand.css" type="text/css">
    <link rel="stylesheet" href="/static/pygments.css" type="text/css">
    <link rel="shortcut icon" type="image/x-icon" href="/static/favicon.ico">
    <script type="text/javascript" src="/static/pinnwand.js"></script>
  </head>
  <body>
    <header>
      <nav>
        <ul>
          <li>
            <a href="/"><img class="logo" src="/static/logo.png"></a>
          </li>
        </ul>
      </nav>
    </header>
    {% block body %}{% end %}
    <footer>
      {% raw handler.application.configuration.footer %}
    </footer>
  </body>
</html>
