database_uri = "sqlite:///:memory:"
paste_size = 256 * 1024  # in bytes
footer = 'View <a href="//github.com/supakeen/pinnwand" target="_BLANK">source code</a>, the <a href="/removal">removal</a> or <a href="/expiry">expiry</a> stories, or read the <a href="/about">about</a> page.'
page_path = None
page_list = ["about", "removal", "expiry"]
